/*
    belcard_calendar.cpp
    Copyright (C) 2015  Belledonne Communications SARL

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "belcard/belcard.hpp"

using namespace ::std;
using namespace ::belr;
using namespace ::belcard;

shared_ptr<BelCardFBURL> BelCardFBURL::parse(const string &input) {
	return BelCardProperty::parseProperty<BelCardFBURL>("FBURL", input);
}

void BelCardFBURL::setHandlerAndCollectors(Parser<shared_ptr<BelCardGeneric>> *parser) {
	parser->setHandler("FBURL", make_fn(BelCardGeneric::create<BelCardFBURL>))
	    ->setCollector("group", make_sfn(&BelCardProperty::setGroup))
	    ->setCollector("any-param", make_sfn(&BelCardProperty::addParam))
	    ->setCollector("VALUE-param", make_sfn(&BelCardProperty::setValueParam))
	    ->setCollector("PID-param", make_sfn(&BelCardProperty::setParamIdParam))
	    ->setCollector("PREF-param", make_sfn(&BelCardProperty::setPrefParam))
	    ->setCollector("TYPE-param", make_sfn(&BelCardProperty::setTypeParam))
	    ->setCollector("MEDIATYPE-param", make_sfn(&BelCardProperty::setMediaTypeParam))
	    ->setCollector("ALTID-param", make_sfn(&BelCardProperty::setAlternativeIdParam))
	    ->setCollector("FBURL-value", make_sfn(&BelCardProperty::setValue));
}

BelCardFBURL::BelCardFBURL() : BelCardProperty() {
	setName("FBURL");
}

shared_ptr<BelCardCALADRURI> BelCardCALADRURI::parse(const string &input) {
	return BelCardProperty::parseProperty<BelCardCALADRURI>("CALADRURI", input);
}

void BelCardCALADRURI::setHandlerAndCollectors(Parser<shared_ptr<BelCardGeneric>> *parser) {
	parser->setHandler("CALADRURI", make_fn(BelCardGeneric::create<BelCardCALADRURI>))
	    ->setCollector("group", make_sfn(&BelCardProperty::setGroup))
	    ->setCollector("any-param", make_sfn(&BelCardProperty::addParam))
	    ->setCollector("VALUE-param", make_sfn(&BelCardProperty::setValueParam))
	    ->setCollector("PID-param", make_sfn(&BelCardProperty::setParamIdParam))
	    ->setCollector("PREF-param", make_sfn(&BelCardProperty::setPrefParam))
	    ->setCollector("TYPE-param", make_sfn(&BelCardProperty::setTypeParam))
	    ->setCollector("MEDIATYPE-param", make_sfn(&BelCardProperty::setMediaTypeParam))
	    ->setCollector("ALTID-param", make_sfn(&BelCardProperty::setAlternativeIdParam))
	    ->setCollector("CALADRURI-value", make_sfn(&BelCardProperty::setValue));
}

BelCardCALADRURI::BelCardCALADRURI() : BelCardProperty() {
	setName("CALADRURI");
}

shared_ptr<BelCardCALURI> BelCardCALURI::parse(const string &input) {
	return BelCardProperty::parseProperty<BelCardCALURI>("CALURI", input);
}

void BelCardCALURI::setHandlerAndCollectors(Parser<shared_ptr<BelCardGeneric>> *parser) {
	parser->setHandler("CALURI", make_fn(BelCardGeneric::create<BelCardCALURI>))
	    ->setCollector("group", make_sfn(&BelCardProperty::setGroup))
	    ->setCollector("any-param", make_sfn(&BelCardProperty::addParam))
	    ->setCollector("VALUE-param", make_sfn(&BelCardProperty::setValueParam))
	    ->setCollector("PID-param", make_sfn(&BelCardProperty::setParamIdParam))
	    ->setCollector("PREF-param", make_sfn(&BelCardProperty::setPrefParam))
	    ->setCollector("TYPE-param", make_sfn(&BelCardProperty::setTypeParam))
	    ->setCollector("MEDIATYPE-param", make_sfn(&BelCardProperty::setMediaTypeParam))
	    ->setCollector("ALTID-param", make_sfn(&BelCardProperty::setAlternativeIdParam))
	    ->setCollector("CALURI-value", make_sfn(&BelCardProperty::setValue));
}

BelCardCALURI::BelCardCALURI() : BelCardProperty() {
	setName("CALURI");
}